/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
package fr.bigeon.gclc;

import java.io.IOException;

import fr.bigeon.gclc.command.Command;
import fr.bigeon.gclc.command.ExitCommand;
import fr.bigeon.gclc.command.HelpExecutor;
import fr.bigeon.gclc.command.ICommandProvider;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.InvalidCommandName;
import fr.bigeon.gclc.manager.ConsoleInput;
import fr.bigeon.gclc.manager.ConsoleOutput;

/** A test-purpose application
 *
 * @author Emmanuel Bigeon */
public class ConsoleTestApplication implements ApplicationAttachement {

    /** Exit command */
    public static final String EXIT = "exit"; //$NON-NLS-1$
    /** Two seconds in milliseconds */
    protected static final long TWO_SECONDS = 2000;

    /***/
    @Override
    @SuppressWarnings("nls")
    public void attach(final ICommandProvider application) {
        try {
            application.add(new ExitCommand(EXIT, (ConsoleApplication) application));
            application.add(new HelpExecutor("help",
                    ((ConsoleApplication) application).root));
            application.add(new Command("test") {

                /* (non-Javadoc)
                 * @see fr.bigeon.gclc.command.ICommand#execute(fr.bigeon.gclc.
                 * manager.ConsoleOutput, fr.bigeon.gclc.manager.ConsoleInput,
                 * java.lang.String[]) */
                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    try {
                        out.println("Test command ran fine");
                    } catch (final IOException e) {
                        throw new CommandRunException("manager closed", e,
                                this);
                    }
                }

                @Override
                public String tip() {
                    return "A test command";
                }

                @Override
                protected String usageDetail() {
                     return null;
                }
            });
            application.add(new Command("long") {
                /* (non-Javadoc)
                 * @see fr.bigeon.gclc.command.ICommand#execute(fr.bigeon.gclc.
                 * manager.ConsoleOutput, fr.bigeon.gclc.manager.ConsoleInput,
                 * java.lang.String[]) */
                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    try {
                        out.println("Waita minute");
                        Thread.sleep(TWO_SECONDS);
                        out.println("done!");
                    } catch (final IOException e) {
                        throw new CommandRunException("manager closed", e,
                                this);
                    } catch (final InterruptedException e) {
                        throw new CommandRunException("wait interrupted", e,
                                this);
                    }
                }

                @Override
                public String tip() {
                    return "A long execution command";
                }

                @Override
                protected String usageDetail() {
                     return null;
                }
            });
            application.add(new Command("failingCmd") {
                /* (non-Javadoc)
                 * @see fr.bigeon.gclc.command.ICommand#execute(fr.bigeon.gclc.
                 * manager.ConsoleOutput, fr.bigeon.gclc.manager.ConsoleInput,
                 * java.lang.String[]) */
                @Override
                public void execute(final ConsoleOutput out,
                                    final ConsoleInput in,
                                    final String... args) throws CommandRunException {
                    throw new CommandRunException("Failing command", this);
                }

                @Override
                public String tip() {
                    return "A long execution command";
                }

                @Override
                protected String usageDetail() {
                     return null;
                }
            });
        } catch (final InvalidCommandName e) {
            e.printStackTrace();
        }
    }
}
